@@ -1,7 +1,7 @@
 // synthv1_lv2.cpp
 //
 /****************************************************************************
-   Copyright (C) 2012-2019, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2020, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -53,6 +53,13 @@
 #define LV2_STATE__StateChanged LV2_STATE_PREFIX "StateChanged"
 #endif
 
+#ifndef LV2_ATOM__PortEvent
+#define LV2_ATOM__PortEvent LV2_ATOM_PREFIX "PortEvent"
+#endif
+#ifndef LV2_ATOM__portTuple
+#define LV2_ATOM__portTuple LV2_ATOM_PREFIX "portTuple"
+#endif
+
 #include <stdlib.h>
 #include <math.h>
 
@@ -67,6 +74,10 @@
 // atom-like message used internally with worker/schedule
 typedef struct {
 	LV2_Atom atom;
+	union {
+		uint32_t    key;
+		const char *path; // not used
+	} data;
 } synthv1_lv2_worker_message;
 
 
@@ -113,6 +124,12 @@ synthv1_lv2::synthv1_lv2 (
 					m_urid_map->handle, LV2_ATOM__Bool);
 				m_urids.atom_Path = m_urid_map->map(
 					m_urid_map->handle, LV2_ATOM__Path);
+			#ifdef CONFIG_LV2_PORT_EVENT
+				m_urids.atom_PortEvent = m_urid_map->map(
+					m_urid_map->handle, LV2_ATOM__PortEvent);
+				m_urids.atom_portTuple = m_urid_map->map(
+					m_urid_map->handle, LV2_ATOM__portTuple);
+			#endif
 				m_urids.time_Position = m_urid_map->map(
 					m_urid_map->handle, LV2_TIME__Position);
 				m_urids.time_beatsPerMinute = m_urid_map->map(
@@ -134,10 +151,6 @@ synthv1_lv2::synthv1_lv2 (
 					m_urid_map->handle, LV2_PATCH__Get);
 				m_urids.patch_Set = m_urid_map->map(
 					m_urid_map->handle, LV2_PATCH__Set);
-				m_urids.patch_Put = m_urid_map->map(
-					m_urid_map->handle, LV2_PATCH__Put);
-				m_urids.patch_body = m_urid_map->map(
-					m_urid_map->handle, LV2_PATCH__body);
 				m_urids.patch_property = m_urid_map->map(
 					m_urid_map->handle, LV2_PATCH__property);
 				m_urids.patch_value = m_urid_map->map(
@@ -328,8 +341,14 @@ void synthv1_lv2::run ( uint32_t nframes )
 				}
 				else
 				if (object->body.otype == m_urids.patch_Get) {
-					// put all property values (probably to UI)
-					patch_put(ndelta);
+					// get one or all property values (probably to UI)...
+					const LV2_Atom_URID *prop = nullptr;
+					lv2_atom_object_get(object,
+						m_urids.patch_property, (const LV2_Atom *) &prop, 0);
+					if (prop && prop->atom.type == m_forge.URID)
+						patch_get(prop->body);
+					else
+						patch_get(0); // all
 				}
 			#endif	// CONFIG_LV2_PATCH
 			}
@@ -374,7 +393,7 @@ void synthv1_lv2::qapp_instantiate (void)
 {
 	if (qApp == nullptr && g_qapp_instance == nullptr) {
 		static int s_argc = 1;
-		static const char *s_argv[] = { __func__, nullptr };
+		static const char *s_argv[] = { SYNTHV1_TITLE, nullptr };
 		g_qapp_instance = new QApplication(s_argc, (char **) s_argv);
 	}
 
@@ -566,6 +585,37 @@ void synthv1_lv2::updatePreset ( bool /*bDirty*/ )
 }
 
 
+void synthv1_lv2::updateParam ( synthv1::ParamIndex index )
+{
+#ifdef CONFIG_LV2_PORT_EVENT
+	if (m_schedule) {
+		synthv1_lv2_worker_message mesg;
+		mesg.atom.type = m_urids.atom_PortEvent;
+		mesg.atom.size = sizeof(mesg.data.key);
+		mesg.data.key  = uint32_t(index);
+		m_schedule->schedule_work(
+			m_schedule->handle, sizeof(mesg), &mesg);
+	}
+#else
+	(void) index; // STFU dang compiler!
+#endif
+}
+
+
+void synthv1_lv2::updateParams (void)
+{
+#ifdef CONFIG_LV2_PORT_EVENT
+	if (m_schedule) {
+		synthv1_lv2_worker_message mesg;
+		mesg.atom.type = m_urids.atom_PortEvent;
+		mesg.atom.size = 0; // nothing else matters.
+		m_schedule->schedule_work(
+			m_schedule->handle, sizeof(mesg), &mesg);
+	}
+#endif
+}
+
+
 void synthv1_lv2::updateTuning (void)
 {
 	if (m_schedule) {
@@ -586,16 +636,10 @@ bool synthv1_lv2::worker_work ( const void *data, uint32_t size )
 	const synthv1_lv2_worker_message *mesg
 		= (const synthv1_lv2_worker_message *) data;
 
-	return (mesg->atom.type == m_urids.state_StateChanged);
-	if (mesg->atom.type == m_urids.state_StateChanged)
-		return true;
-	else
-	if (mesg->atom.type == m_urids.tun1_update) {
+	if (mesg->atom.type == m_urids.tun1_update)
 		synthv1::resetTuning();
-		return true;
-	}
 
-	return false;
+	return true;
 }
 
 
@@ -607,11 +651,20 @@ bool synthv1_lv2::worker_response ( const void *data, uint32_t size )
 	const synthv1_lv2_worker_message *mesg
 		= (const synthv1_lv2_worker_message *) data;
 
+#ifdef CONFIG_LV2_PORT_EVENT
+	if (mesg->atom.type == m_urids.atom_PortEvent) {
+		if (mesg->atom.size > 0)
+			return port_event(synthv1::ParamIndex(mesg->data.key));
+		else
+			return port_events();
+	}
+	else
+#endif
 	if (mesg->atom.type == m_urids.state_StateChanged)
 		return state_changed();
 
 #ifdef CONFIG_LV2_PATCH
-	return patch_put(m_ndelta, mesg->atom.type);
+	return patch_get(mesg->atom.type);
 #else
 	return true;
 #endif
@@ -632,58 +685,115 @@ bool synthv1_lv2::state_changed (void)
 
 #ifdef CONFIG_LV2_PATCH
 
-bool synthv1_lv2::patch_put ( uint32_t ndelta, uint32_t type )
+bool synthv1_lv2::patch_set ( LV2_URID key )
 {
 	static char s_szNull[1] = {'\0'};
 
-	if (type == m_urids.patch_Put)
-		type = 0;
-
-	lv2_atom_forge_frame_time(&m_forge, ndelta);
+	lv2_atom_forge_frame_time(&m_forge, m_ndelta);
 
 	LV2_Atom_Forge_Frame patch_frame;
-	lv2_atom_forge_object(&m_forge, &patch_frame, 0, m_urids.patch_Put);
-	lv2_atom_forge_key(&m_forge, m_urids.patch_body);
+	lv2_atom_forge_object(&m_forge, &patch_frame, 0, m_urids.patch_Set);
 
-	LV2_Atom_Forge_Frame body_frame;
-	lv2_atom_forge_object(&m_forge, &body_frame, 0, 0);
+	lv2_atom_forge_key(&m_forge, m_urids.patch_property);
+	lv2_atom_forge_urid(&m_forge, key);
 
-	if (type == 0 || type == m_urids.p201_tuning_enabled) {
-		lv2_atom_forge_key(&m_forge, m_urids.p201_tuning_enabled);
+	lv2_atom_forge_key(&m_forge, m_urids.patch_value);
+	if (key == m_urids.p201_tuning_enabled)
 		lv2_atom_forge_bool(&m_forge, synthv1::isTuningEnabled());
-	}
-	if (type == 0 || type == m_urids.p202_tuning_refPitch) {
-		lv2_atom_forge_key(&m_forge, m_urids.p202_tuning_refPitch);
+	else
+	if (key == m_urids.p202_tuning_refPitch)
 		lv2_atom_forge_float(&m_forge, synthv1::tuningRefPitch());
-	}
-	if (type == 0 || type == m_urids.p203_tuning_refNote) {
-		lv2_atom_forge_key(&m_forge, m_urids.p203_tuning_refNote);
+	else
+	if (key == m_urids.p203_tuning_refNote)
 		lv2_atom_forge_int(&m_forge, synthv1::tuningRefNote());
-	}
-	if (type == 0 || type == m_urids.p204_tuning_scaleFile) {
+	else
+	if (key == m_urids.p204_tuning_scaleFile) {
 		const char *pszScaleFile = synthv1::tuningScaleFile();
 		if (pszScaleFile == nullptr)
 			pszScaleFile = s_szNull;
-		lv2_atom_forge_key(&m_forge, m_urids.p204_tuning_scaleFile);
 		lv2_atom_forge_path(&m_forge, pszScaleFile, ::strlen(pszScaleFile) + 1);
 	}
-	if (type == 0 || type == m_urids.p205_tuning_keyMapFile) {
+	else
+	if (key == m_urids.p205_tuning_keyMapFile) {
 		const char *pszKeyMapFile = synthv1::tuningKeyMapFile();
 		if (pszKeyMapFile == nullptr)
 			pszKeyMapFile = s_szNull;
-		lv2_atom_forge_key(&m_forge, m_urids.p205_tuning_keyMapFile);
 		lv2_atom_forge_path(&m_forge, pszKeyMapFile, ::strlen(pszKeyMapFile) + 1);
 	}
 
-	lv2_atom_forge_pop(&m_forge, &body_frame);
 	lv2_atom_forge_pop(&m_forge, &patch_frame);
 
 	return true;
 }
 
+bool synthv1_lv2::patch_get ( LV2_URID key )
+{
+	if (key == 0 || key == m_urids.tun1_update) {
+		patch_set(m_urids.p201_tuning_enabled);
+		patch_set(m_urids.p202_tuning_refPitch);
+		patch_set(m_urids.p203_tuning_refNote);
+		patch_set(m_urids.p204_tuning_scaleFile);
+		patch_set(m_urids.p205_tuning_keyMapFile);
+		if (key) return true;
+	}
+
+	if (key) patch_set(key);
+
+	return true;
+}
+
 #endif	// CONFIG_LV2_PATCH
 
 
+#ifdef CONFIG_LV2_PORT_EVENT
+
+bool synthv1_lv2::port_event ( synthv1::ParamIndex index )
+{
+	lv2_atom_forge_frame_time(&m_forge, m_ndelta);
+
+	LV2_Atom_Forge_Frame obj_frame;
+	lv2_atom_forge_object(&m_forge, &obj_frame, 0, m_urids.atom_PortEvent);
+	lv2_atom_forge_key(&m_forge, m_urids.atom_portTuple);
+
+	LV2_Atom_Forge_Frame tup_frame;
+	lv2_atom_forge_tuple(&m_forge, &tup_frame);
+
+	lv2_atom_forge_int(&m_forge, int32_t(ParamBase + index));
+	lv2_atom_forge_float(&m_forge, synthv1::paramValue(index));
+
+	lv2_atom_forge_pop(&m_forge, &tup_frame);
+	lv2_atom_forge_pop(&m_forge, &obj_frame);
+
+	return true;
+}
+
+
+bool synthv1_lv2::port_events (void)
+{
+	lv2_atom_forge_frame_time(&m_forge, m_ndelta);
+
+	LV2_Atom_Forge_Frame obj_frame;
+	lv2_atom_forge_object(&m_forge, &obj_frame, 0, m_urids.atom_PortEvent);
+	lv2_atom_forge_key(&m_forge, m_urids.atom_portTuple);
+
+	LV2_Atom_Forge_Frame tup_frame;
+	lv2_atom_forge_tuple(&m_forge, &tup_frame);
+
+	for (uint32_t i = 0; i < synthv1::NUM_PARAMS; ++i) {
+		synthv1::ParamIndex index = synthv1::ParamIndex(i);
+		lv2_atom_forge_int(&m_forge, int32_t(ParamBase + index));
+		lv2_atom_forge_float(&m_forge, synthv1::paramValue(index));
+	}
+
+	lv2_atom_forge_pop(&m_forge, &tup_frame);
+	lv2_atom_forge_pop(&m_forge, &obj_frame);
+
+	return true;
+}
+
+#endif	// CONFIG_LV2_PORT_EVENT
+
+
 //-------------------------------------------------------------------------
 // synthv1_lv2 - LV2 desc.
 //
